
import setuptools


with open('README.md') as f:
    README = f.read()

setuptools.setup(
    author="Rahul Kumar",
    author_email="kum28ra@gmail.com",
    name='pyaudiobook',
    license="MIT",
    description='pyaudiobook is a python package which converts pdf to mp3 files',
    version='v0.0.5',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/Rahul-157/AudioBook',
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=['gtts','pdfminer'],
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Education',
        'Topic :: Multimedia :: Sound/Audio',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
    ],
)


#!/usr/bin/env python
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.pdfpage import PDFPage
from pdfminer.converter import  TextConverter
from pdfminer.layout import LAParams
from io import StringIO
from gtts import gTTS
from os import stat
from re import sub
from threading import Thread
from .utils import *
from time import sleep,time


done = False

def pdfParser(data,filename):
    fp = open(data, 'rb')
    rsrcmgr = PDFResourceManager()
    retstr = StringIO()
    laparams = LAParams()
    device = TextConverter(rsrcmgr, retstr,  laparams=laparams)
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    for page in PDFPage.get_pages(fp):
        interpreter.process_page(page)
        data =  retstr.getvalue() 
    data = sub(" \(cid:\d{1,6}\) ","",data)
    text_file = open(filename,mode="w+", encoding="utf-8")
    text_file.write(data)
    text_file.close()
  
def worker(text,filename,i,language):
    tts = gTTS(text,lang=language)
    filename = filename.replace(".txt", "")
    tts.save(filename+"_"+str(i)+".mp3")

def collectFiles(filename,output_name):
    filename = filename.replace(".txt", "")
    fp = open(filename+"_0.mp3",mode="rb")
    book = fp.read()
    fp.close()
    for i in range(1,getThreads()):
        fp= open(filename+"_"+str(i)+".mp3",mode="rb")
        book = book + fp.read()
        fp.close()
    opt_file = open(output_name,mode="wb")
    opt_file.write(book)
    opt_file.close()


def processFile(text_file,lang):
    global done
    f = open(text_file,mode="r", encoding="utf-8")
    text = f.read()
    f.close()
    block_size = int(len(text)/getThreads())+1
    threads = list()
    start=0
    s_time = time()
    for i in range(getThreads()):
        last_idx = findLastWord(start,block_size,text)
        if(i==getThreads()-1):
            last_idx=len(text)
        threads.append(Thread(target = worker,args=(text[start:last_idx+1],text_file,i,lang)))
        threads[i].start()
        start=last_idx+1
    progressBar(text_file)
    for i in range(getThreads()):
        threads[i].join()
    done = True
    e_time = time()
    print(f'\rProgress: |{"█"*70}| {100.0}% Completed')
    logger.info("Conversion Finished in : "+ str(e_time-s_time) +" seconds\n")


def printProgressBar (total, filename,length = 70, fill = '█'):
    global done
    while not done:
        itr=0
        for i in range(getThreads()):
            itr = itr + (stat(filename+"_"+str(i)+".mp3").st_size)
        if itr >= total: 
            break
        percent = ("{0:.1f}").format(100 * (itr / float(total)))
        filledLength = int(length * itr // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        print(f'\rProgress: |{bar}| {percent}% Completed', end = '\r')
        sleep(5)

def progressBar(text_file):
    total = 916.55 * stat(text_file).st_size
    text_file = text_file.replace(".txt","")
    print(f'Progress: |{"-"*70}| {0.0}% Completed', end = '\r')
    p_bar = Thread(target = printProgressBar,args=(total,text_file))
    p_bar.start()

   


from .audioBook import removeExtraPages,collectFiles,processFile,pdfParser
from .utils import *
from os.path import expanduser,join,exists,abspath 
from sys import platform as PLATFORM
from datetime import datetime

def setDefaultsIfBlank(lang,op,on):
    if not lang:
        lang="en"
    if not op:
        op = expanduser('~')
    else:
        op = getAbsolutePath(op)
    if not on:
        on = "Output_"+datetime.now().strftime("%X").replace(":","_")
    return lang,join(op,on)

class Pdf2Mp3:
    def __init__(self,filename, lang="", out_path="",out_name="",num_threads=""):
        assert filename, "Input Pdf File is required"
        assert filename.split(".")[-1]=="pdf", "Only PDF file is accepted"
        #Todo :  LanguageCheck
        lang,output = setDefaultsIfBlank(lang, out_path, out_name)
        lang = languageCheck(lang)
        self.input = getAbsolutePath(filename)
        assert exists(self.input), "File not found"
        self.lang = lang
        self.output = output+".mp3"
        self.text_file = join(TEMP_LOC,randomName()+".txt")
        self.threads = setThreads(num_threads)
        if not exists(abspath(TEMP_LOC)):
            runCommand(["mkdir " , TEMP_LOC])
    
    def convert(self):
        logger.info("Generating Text...")
        self.generateText()
        logger.info("Text file : "+ self.text_file)
        removeExtraPages(self.text_file)
        logger.info("Conversion Started...")
        processFile(self.text_file,self.lang)   
        logger.info("Wrapping Up...")
        collectFiles(self.text_file,self.output)
        cleanup()
        logger.info("Finished...")
        logger.info("Output File :"+self.output)


    def generateText(self):
        if("linux" in PLATFORM):
            runCommand(["less ",self.input," > ",self.text_file])
        else:
            pdfParser(self.input,self.text_file)
        # Extract Text from Pdf
        f = open(self.text_file)
        text = f.read()
        f.close()
        return text

__all__= ["Pdf2Mp3"]


from random import random
import ctypes 
from gtts import langs
from sys import platform as PLATFORM
from os import system,path
import logging
from multiprocessing import cpu_count
logger = logging.getLogger('pyaudiobook')
logging.basicConfig(level = logging.INFO)

_THREADS = 1
NOTIFY_MSG = "'Remove Pages Contents, Preface, Acknowledgements, References'"
LANGS = langs._langs
TEMP_LOC = path.join(path.expanduser("~"),".TXTnMP3")


def setThreads(num_threads):
    global _THREADS
    if not num_threads:
        _THREADS = cpu_count()
    else:
        assert int(th)>0, "Number of Threads Should be Greater than 0"
        _THREADS = int(th)

def getThreads():
    return _THREADS

def runCommand(cmd_list):
    command = " ".join(cmd_list)
    exit_code = system(command)
    assert exit_code == 0, "Command : " + command + "Failed | ExitCode : " + str(exit_code)
    return exit_code

def getAbsolutePath(filepath):
    if "linux" not in PLATFORM:
        filepath = filepath.replace('/','\\')
    if(filepath[0]=='~'):
        filepath = path.join(path.expanduser('~'),filepath[2:])
    else:
        filepath = path.abspath(filepath)
    return filepath
       

def languageCheck(language):
    while(language not in LANGS):
        logger.error("Language Not supported... Default : en English\nSupport Lanaguages are : \n")
        #Todo : Print only once
        for key,value in LANGS.items():
            print(key + " : "+ value)
        language = input("Enter Language Key eg: en for English\n")
    return language

def findLastWord(start,blk_size,text):
    lst_idx = start + blk_size
    if(lst_idx>=len(text)):
        lst_idx=len(text)-1
    for i in range(lst_idx,start,-1):
        if(text[i]==" "):
            return i
            break
  
def cleanup():
    if "linux" in PLATFORM:
        runCommand(["rm -rf ",TEMP_LOC])
    else:
        runCommand(["rd /s /q ",TEMP_LOC])


def randomName(length=6):
    s=""
    a="abcdfeghijklmnopqrstuvwxyz1234567890"
    for i in range(length):
        s+=a[int(random()*100)%36]
    return s

def removeExtraPages(filename):
    editors_nix = ["vim","gedit","nano","vi"]
    if "linux" in PLATFORM:
        for editor in editors_nix:
            try:
                runCommand(["notify-send", NOTIFY_MSG])
                ret_val = runCommand([editor,filename])
                if not ret_val:
                    break
            except:
                pass
    else:    
        ctypes.windll.user32.MessageBoxW(0, NOTIFY_MSG, "Notification", 1)
        runCommand(["notepad",filename])